<?php

/**
 * ModelesController
 *
 * web-DPO : Outil de gestion de vos traitements dans le cadre de la 
 * réglementation relative à la protection des données personnelles (RGPD)
 * 
 * Copyright (c) Libriciel SCOP (https://www.libriciel.fr/)
 *
 * Licensed under the GNU Affero General Public License version 3 License - AGPL v3
 * For full copyright and license information, please see the "LICENSE" file.
 * Redistributions of files must retain the above copyright notice.
 * 
 * @copyright   Copyright (c) Libriciel SCOP (https://www.libriciel.fr/)
 * @link        https://www.libriciel.fr/web-dpo/
 * @since       web-DPO v1.0.0
 * @license     [GNU Affero General Public License version 3](http://www.gnu.org/licenses/agpl-3.0.html) - AGPL v3
 * @version     v1.0.0
 * @package     App.Controller
 */

App::uses('Folder', 'Utility');
App::uses('File', 'Utility');
App::uses('ListeDroit', 'Model');

class ModelesController extends AppController {

    public $uses = [
        'Modele',
        'Formulaire',
        'Champ',
        'Fiche',
        'Valeur',
        'Organisation',
        'User',
        'Responsable',
        'Coresponsable',
        'Soustraitant',
        'Soustraitance'
    ];

    /**
     * @access public
     * @created 18/06/2015
     * @version V1.0.0
     */
    public function index()
    {
        if (true !== $this->Droits->authorized([ListeDroit::GESTION_MODELE])) {
            throw new ForbiddenException(__d('default', 'default.flasherrorPasDroitPage'));
        }

        // Superadmin non autorisé
        if (true == ($this->Droits->authorized($this->Droits->isSu()))) {
            throw new ForbiddenException(__d('default', 'default.flasherrorPasDroitPage'));
        }
        
        $this->set('title', __d('modele', 'modele.titreListeModele'));
        
        $modeles = $this->Formulaire->find('all', [
            'contain' => [
                'Modele'
            ],
            'conditions' => [
                'organisations_id' => $this->Session->read('Organisation.id')
            ],
            'order' => [
                'libelle ASC'
            ]
        ]);
        
        foreach ($modeles as $keys => $modele) {
            if (!file_exists(CHEMIN_MODELES.$modele['Modele']['fichier'])) {
                $this->Modele->delete($modele['Modele']['id']);
                
                foreach ($modele['Modele'] as $key => $value) {
                    $modele['Modele'][$key] = null;
                }
                
                $modeles[$keys]['Modele'] = $modele['Modele'];
            }
        }
        
        $this->set(compact('modeles'));
    }

    /**
     * @access public
     * @created 18/06/2015
     * @version V1.0.0
     */
    public function add()
    {
        if (true !== $this->Droits->authorized([ListeDroit::GESTION_MODELE])) {
            throw new ForbiddenException(__d('default', 'default.flasherrorPasDroitPage'));
        }
        
        // Superadmin non autorisé
        if (true == ($this->Droits->authorized($this->Droits->isSu()))) {
            throw new ForbiddenException(__d('default', 'default.flasherrorPasDroitPage'));
        }

        $this->Modele->begin();
        $saveFile = $this->Modele->saveFile($this->request->data, $this->request->data['Modele']['idUploadModele']);

        if ($saveFile == 0) {
            $this->Modele->commit();
            $this->Session->setFlash(__d('modele', 'modele.flashsuccessModeleEnregistrer'), 'flashsuccess');
        } elseif ($saveFile == 1) {
            $this->Modele->rollback();
            $this->Session->setFlash(__d('modele', 'modele.flasherrorFicherTropLourd'), 'flasherror');
        } elseif ($saveFile == 2) {
            $this->Modele->rollback();
            $this->Session->setFlash(__d('modele', 'modele.flasherrorExtensionNonValide'), 'flasherror');
        } elseif ($saveFile == 3) {
            $this->Modele->rollback();
            $this->Session->setFlash(__d('modele', 'modele.flashwarningAucunFichier'), 'flashwarning');
        }

        $this->redirect($this->Referers->get());
    }

    /**
     * @param type $file
     * @return type
     * 
     * @access public
     * @created 18/06/2015
     * @version V1.0.0
     */
    public function download($file, $nameFile)
    {
        if (true !== $this->Droits->authorized([ListeDroit::GESTION_MODELE])) {
            throw new ForbiddenException(__d('default', 'default.flasherrorPasDroitPage'));
        }

        $this->response->file(CHEMIN_MODELES . $file, [
            'download' => true,
            'name' => $nameFile
        ]);
        
        return $this->response;
    }

    /**
     * Permet de supprimer en bdd de le model associer par qu'ontre on ne 
     * supprime dans aucun cas le fichier enregistré
     * 
     * @param type $file --> c'est le nom du model (en générale 15614325.odt)
     * qui est enregistré dans app/webroot/files/models
     */
    public function delete($file)
    {
        if (true !== $this->Droits->authorized([ListeDroit::GESTION_MODELE])) {
            throw new ForbiddenException(__d('default', 'default.flasherrorPasDroitPage'));
        }

        $modeles = $this->Modele->find('all', [
            'conditions' => [
                'fichier' => $file
            ]
        ]);
        
        if ($modeles) {
            $isDeleted = $this->Modele->deleteAll([
                'fichier' => $file
            ]);

            if ($isDeleted) {
                $this->Session->setFlash(__d('modele', 'modele.flashsuccessModeleSupprimer'), 'flashsuccess');
            } else {
                $this->Session->setFlash(__d('modele', 'modele.flasherrorErreurSupprimerModele'), 'flasherror');
            }
        } else {
            $this->Session->setFlash(__d('modele', 'modele.flasherrorModeleInexistant'), 'flasherror');
        }

        $this->redirect($this->Referers->get());
    }

    /**
     * 
     * @access public
     * @created 26/04/2016
     * @version V1.0.2
     */
    public function infoVariable($idFormulaire = null)
    {
        if (true !== $this->Droits->authorized([ListeDroit::GESTION_MODELE])) {
            throw new ForbiddenException(__d('default', 'default.flasherrorPasDroitPage'));
        }

        if ($this->Session->read('Organisation.dpo') === null) {
            throw new ForbiddenException(__d('default', 'default.flasherrorPasDroitPage'));
        }
        
        $this->set('title', __d('modele', 'modele.titreInfoVariableModele'));

        //Information sur l'organisation
        $organisation = $this->Organisation->find('first', [
            'conditions' => [
                'id' => $this->Session->read('Organisation.id')
            ],
            'fields' => [
                'raisonsociale',
                'telephone',
                'fax',
                'adresse',
                'email',
                'sigle',
                'siret',
                'ape',
                'dpo',
                'numerodpo'
            ]
        ]);
        
        // Responsable de l'entité
        $responsableEntite = $this->Organisation->find('first', [
            'conditions' => [
                'id' => $this->Session->read('Organisation.id')
            ],
            'fields' => [
                'civiliteresponsable',
                'nomresponsable',
                'prenomresponsable',
                'emailresponsable',
                'telephoneresponsable',
                'fonctionresponsable',
                'responsable_nom_complet',
                'responsable_nom_complet_court'
            ]
        ]);
        
        // Information sur le DPO de l'organisation
        $userDPO = $this->User->find('first', [
            'conditions' => [
                'id' => $organisation['Organisation']['dpo']
            ],
            'fields' => [
                'civilite',
                'nom',
                'prenom',
                'email',
                'telephonefixe',
                'telephoneportable',
                'nom_complet',
                'nom_complet_court'
            ]
        ]);

        if (isset($organisation['Organisation']['dpo']) === true) {
            unset($organisation['Organisation']['dpo']);
        }
        
        if (isset($organisation['Organisation']['numerodpo']) === true) {
            $userDPO['User'] += [
                'numerodpo' => $organisation['Organisation']['numerodpo']
            ];
            
            unset($organisation['Organisation']['numerodpo']);
        }

        $variablesOrganisation = [];
        foreach ($organisation['Organisation'] as $key => $valueOrganisation) {
            $variablesOrganisation['Entite'][$key] = [
                'organisation_'.$key => $valueOrganisation
            ];
        }
        
        foreach ($responsableEntite['Organisation'] as $key => $valueResponsable) {
            $variablesOrganisation['Responsable'][$key] = [
                'organisation_'.$key => $valueResponsable
            ];
        }
        
        foreach ($userDPO['User'] as $key => $valueUserDPO) {
            $variablesOrganisation['Dpo'][$key] = [
                'dpo_'.$key => $valueUserDPO
            ];
        }
        $this->set(compact('variablesOrganisation'));

        $valeurPropreTraitement = [
            'declarant' => [
                'declarantpersonnenom',
                'declarantservice',
                'declarantpersonneemail'
            ],
            'traitement' => [
                'outilnom',
                'finaliteprincipale',
                'transfert_hors_ue',
                'donnees_sensibles',
                'obligation_pia',
                'realisation_pia',
                'depot_pia',
                'coresponsable',
                'soustraitance',
                'numeroenregistrement',
                'norme',
                'normelibelle',
                'normedescription',
                'fichecreated',
                'fichemodified',
            ]
        ];

        $rt_externe = false;
        $viewAnnexe = false;
        $viewHistorique = false;
        if ($idFormulaire != null) {
            $viewAnnexe = true;
            $viewHistorique = true;

            //information sur les champs du formulaire
            $variables = $this->Champ->find('all', [
                'conditions' => [
                    'formulaire_id' => $idFormulaire,
                    'champ_coresponsable' => false,
                    'champ_soustraitant' => false,
                ],
                'fields' => [
                    'type',
                    'details'
                ]
            ]);
            $variables = Hash::extract($variables, '{n}.Champ');

            $this->_getCoresponsables($idFormulaire);
            $this->_getSoustraitances($idFormulaire);

            $optionsUseFormulaire = $this->Formulaire->find('first', [
                'conditions' => [
                    'id' => $idFormulaire
                ],
                'fields' => [
                    'usesousfinalite',
                    'usebaselegale',
                    'usedecisionautomatisee',
                    'usetransferthorsue',
                    'usedonneessensible',
                    'usepia'
                ]
            ]);

            $formulaireRT = $this->Formulaire->find('first', [
                'conditions' => [
                    'id' => $idFormulaire
                ],
                'fields' => [
                    'rt_externe'
                ]
            ]);
            $rt_externe = $formulaireRT['Formulaire']['rt_externe'];

            $prefixChampName = 'rt_organisation_';
            $rtVariables = [];
            if ($formulaireRT['Formulaire']['rt_externe'] === true) {
                $rtVariables = [
                    'rt_externe_raisonsociale',
                    'rt_externe_telephone',
                    'rt_externe_fax',
                    'rt_externe_adresse',
                    'rt_externe_email',
                    'rt_externe_siret',
                    'rt_externe_ape',
                    'rt_externe_civiliteresponsable',
                    'rt_externe_prenomresponsable',
                    'rt_externe_nomresponsable',
                    'rt_externe_fonctionresponsable',
                    'rt_externe_emailresponsable',
                    'rt_externe_telephoneresponsable',
                    'rt_externe_civility_dpo',
                    'rt_externe_prenom_dpo',
                    'rt_externe_nom_dpo',
                    'rt_externe_numerocnil_dpo',
                    'rt_externe_email_dpo',
                    'rt_externe_telephonefixe_dpo',
                    'rt_externe_telephoneportable_dpo',
                ];

                $prefixChampName = 'st_organisation_';
            }

            $rtOrstValues = $this->Organisation->find('first', [
                'joins' => [
                    $this->Organisation->join('Dpo', ['type' => 'INNER']),
                ],
                'conditions' => [
                    'Organisation.id' => $this->Session->read('Organisation.id'),
                    'Dpo.id = Organisation.dpo'
                ],
                'fields' => [
                    'Organisation.raisonsociale',
                    'Organisation.telephone',
                    'Organisation.fax',
                    'Organisation.adresse',
                    'Organisation.email',
                    'Organisation.sigle',
                    'Organisation.siret',
                    'Organisation.ape',
                    'Organisation.civiliteresponsable',
                    'Organisation.prenomresponsable',
                    'Organisation.nomresponsable',
                    'Organisation.emailresponsable',
                    'Organisation.telephoneresponsable',
                    'Organisation.fonctionresponsable',
                    'Organisation.numerodpo',
                    $this->Organisation->Dpo->vfNomComplet(),
                    'Dpo.email',
                    'Dpo.telephonefixe',
                    'Dpo.telephoneportable',
                ]
            ]);

            $arrayRtOrSt = [];
            foreach ($rtOrstValues['Organisation'] as $key => $value) {
                $arrayRtOrSt[] = $prefixChampName . $key;
            }

            foreach ($rtOrstValues['Dpo'] as $key => $value) {
                $arrayRtOrSt[] = $prefixChampName . $key . '_dpo';
            }

            $this->set(compact('variables', 'optionsUseFormulaire', 'rtVariables', 'arrayRtOrSt'));
        }

        $this->set(compact('valeurPropreTraitement', 'rt_externe', 'viewAnnexe', 'viewHistorique'));
    }

    private function _getCoresponsables($formulaire_id)
    {
        $coresponsables =  array_filter(array_keys($this->Responsable->schema()), function($name) {
            return in_array($name, ['id', 'createdbyorganisation', 'created', 'modified']) === false && preg_match('/_id$/', $name) !== 1;
        });

        foreach ($coresponsables as $key => $coresponsable) {
            $translation = str_replace('_', '', ucwords($coresponsable, "_"));
            $coresponsables[__d('responsable_soustraitant', 'responsable_soustraitant.champ' . $translation)] = 'coresponsable_'.$coresponsable;
            unset($coresponsables[$key]);
        }

        $coresponsableFields = $this->Champ->find('all', [
            'conditions' => [
                'formulaire_id' => $formulaire_id,
                'champ_coresponsable' => true,
                'champ_soustraitant' => false,
            ],
            'fields' => [
                'type',
                'details'
            ]
        ]);

        foreach ($coresponsableFields as $coresponsableField) {
            $details = json_decode($coresponsableField['Champ']['details'], true);
            $coresponsables[$details['label']] = $details['name'];
        }

        $this->set(compact('coresponsables'));
    }

    private function _getSoustraitances($formulaire_id)
    {
        $soustraitances =  array_filter(array_keys($this->Soustraitant->schema()), function($name) {
            return in_array($name, ['id', 'createdbyorganisation', 'created', 'modified']) === false && preg_match('/_id$/', $name) !== 1;
        });

        foreach ($soustraitances as $key => $soustraitance) {
            $translation = str_replace('_', '', ucwords($soustraitance, "_"));
            $soustraitances[__d('responsable_soustraitant', 'responsable_soustraitant.champ' . $translation)] = 'soustraitance_'.$soustraitance;
            unset($soustraitances[$key]);
        }

        $soustraitanceFields = $this->Champ->find('all', [
            'conditions' => [
                'formulaire_id' => $formulaire_id,
                'champ_coresponsable' => false,
                'champ_soustraitant' => true,
            ],
            'fields' => [
                'type',
                'details'
            ]
        ]);

        foreach ($soustraitanceFields as $soustraitanceField) {
            $details = json_decode($soustraitanceField['Champ']['details'], true);
            $soustraitances[$details['label']] = $details['name'];
        }

        $this->set(compact('soustraitances'));
    }

}
