<?php

/**
 * WorkflowsController
 *
 * web-DPO : Outil de gestion de vos traitements dans le cadre de la
 * réglementation relative à la protection des données personnelles (RGPD)
 *
 * Copyright (c) Libriciel SCOP (https://www.libriciel.fr/)
 *
 * Licensed under the GNU Affero General Public License version 3 License - AGPL v3
 * For full copyright and license information, please see the "LICENSE" file.
 * Redistributions of files must retain the above copyright notice.
 *
 * @copyright   Copyright (c) Libriciel SCOP (https://www.libriciel.fr/)
 * @link        https://www.libriciel.fr/web-dpo/
 * @since       web-DPO v1.0.0
 * @license     [GNU Affero General Public License version 3](http://www.gnu.org/licenses/agpl-3.0.html) - AGPL v3
 * @version     v2.1.0
 * @package     App.Controller
 */

App::uses('ListeDroit', 'Model');

class WorkflowsController extends AppController
{
    public $uses = [
        'OrganisationUserService',
        'Service',
        'Workflow'
    ];

    /**
     * Vérification de l'accès aux actions en fonction du profil de l'utilisateur connecté.
     * Les vérifications de l'accès aux enregistrements se font dans les méthodes d'actions.
     */
    public function beforeFilter() {
        parent::beforeFilter();

        $action = Inflector::underscore($this->request->params['action']);
//        $listeDroitIndex = [ListeDroit::CREER_SERVICE, ListeDroit::MODIFIER_SERVICE, ListeDroit::SUPPRIMER_SERVICE];
//
//        if ($action === 'add') {
//            $this->Droits->assertAuthorized([ListeDroit::CREER_SERVICE]);
//        } elseif ($action === 'edit') {
//            $this->Droits->assertAuthorized([ListeDroit::MODIFIER_SERVICE]);
//        } elseif ($action === 'index') {
//            $this->Droits->assertAuthorized($listeDroitIndex);
//        } elseif ($action === 'delete') {
//            $this->Droits->assertAuthorized([ListeDroit::SUPPRIMER_SERVICE]);
//        } else {
//            throw new ForbiddenException(__d('default', 'default.flasherrorPasDroitPage'));
//        }
    }

    /**
     *
     * @access public
     *
     * @created 15/10/2020
     * @author Théo GUILLON <theo.guillon@libriciel.coop>
     * @version V2.1.0
     */
    public function index()
    {
        $this->set('title', __d('workflow', 'workflow.titreCircuits') . $this->Session->read('Organisation.raisonsociale'));

        $query = [
            'conditions' => [
                'organisation_id' => $this->Session->read('Organisation.id')
            ],
            'order' => 'name ASC',
            'limit' => 20
        ];

        $this->paginate = $query;
        $circuits = $this->paginate('Workflow');

        $this->set(compact('circuits'));
    }

    /**
     *
     * @access public
     *
     * @created 16/10/2020
     * @author Théo GUILLON <theo.guillon@libriciel.coop>
     * @version V2.1.0
     */
    public function add()
    {
        $this->set('title', "Création d'un circuit");

        if ($this->request->is('post') || $this->request->is('put')) {
//            $data = $this->request->data;

//            array (
//                'steps' => array (
//                    0 => array (
//                        'name' => 'Théo',
//                        'users' => array (
//                            0 => 1,
//                        ),
//                        'groups' => '',
//                    ),
//                    1 => array (
//                        'name' => 'Théo + un autre',
//                        'users' => array (
//                            0 => 5,
//                            1 => 3,
//                        ),
//                        'groups' => '',
//                    ),
//                ),
//                'name' => 'Pour théo',
//                'description' => 'Parce que théo c\'est le meilleur ! ',
//            )

            $data = [
                'steps' => [
                    0 => [
                        'name' => 'Théo',
                        'users' => [
                            0 => 2,
                        ],
                        'groups' => '',
                    ],
                ],
                'name' => 'Pour petit théo',
            ];

            $workflow = $this->Workflow->create();

            $deployment = $this->createNewDeployment($workflow, $data);

            if ($deployment === false) {
                return;
            }

            die;
        }
    }

    /**
     * @param $workflow
     * @param $data
     * @return false
     * @throws Exception
     */
    protected function createNewDeployment($workflow, $data)
    {
//        if (!$this->checkData($data, $workflow)) {
//            return false;
//        }

        // Set the right structure and project
        $data['organisation_id'] = $this->Session->read('Organisation.id');

        debug($data);

        // Création et engregistrement d'un fichier bpm20.xml pour l'envoit à flowable
        $dataSendToFlowable = $this->Workflow->convertJsonToXml($data);

        if ($dataSendToFlowable === false) {
            $this->setResponse($this->getResponse()->withStatus(500)); // AFAIRE: changer code d'erreur ?
            $this->set('errors', ['file' => 'error when saving file']); // AFAIRE: changer message d'erreur ?

            throw new Exception('error when saving file'); // AFAIRE: à voir
        }

        //Create the deployment in Flowable
        $deployment = $this->Workflow->createDeployment($dataSendToFlowable);

        // ajout du nom ert description du circuit au données pour le patch
        $deployment['name'] = $data['name'];
        $deployment['description'] = $data['description'] ?? null;
        $deployment['active'] = false; //default to false

        //Insert Flowable id into webActes database
        // AFAIRE: refacto ? en l'état patchEntity avec le deployment est pas super élégant, un generateWorkflowFromDeployment serait pas mal

        return $deployment;
    }

    protected function checkData(array $data): bool
    {
        // Check si le tableau reçu contient les keys "name" et "steps"
        if (empty($data['name'])) {
            $this->setResponse($this->getResponse()->withStatus(400));
            $this->set('errors', ['name' => 'missing or empty value']);

            return false;
        }
        if (empty($data['steps']) || !is_array($data['steps'])) {
            $this->setResponse($this->getResponse()->withStatus(400));
            $this->set('errors', ['steps' => 'missing, empty or invalid value']);

            return false;
        }

        $structureId = $this->Multico->getCurrentStructureId();
        /* @var $StructuresUsers StructuresUsersTable */
        $StructuresUsers = $this->loadModel('StructuresUsers');

        // Check si aucun "name" dans la key "steps" n'est vide
        foreach ($data['steps'] as $idx => $step) {
            if (empty($step['name'])) {
                $this->setResponse($this->getResponse()->withStatus(400));
                $this->set('errors', ['steps' => 'missing name data for step #' . $idx]);

                return false;
            }

            if (empty($step['users'])) {
                $this->setResponse($this->getResponse()->withStatus(400));
                $this->set('errors', ['steps' => 'missing users data for step #' . $idx . ' : ' . $step['name']]);

                return false;
            }

            $userCount = $StructuresUsers->find()
                ->select('StructuresUsers.user_id')
                ->where([
                    'StructuresUsers.structure_id' => $structureId,
                    'StructuresUsers.user_id IN' => $step['users'],
                ])
                ->count();

            if (count($step['users']) !== $userCount) {
                $this->setResponse($this->getResponse()->withStatus(400));
                $this->set('errors', ['steps' => 'invalid users data for step #' . $idx . ' : ' . $step['name']]);

                return false;
            }
        }

        // unicité des noms des étapes
        $names = Hash::extract($data, 'steps.{n}.name');
        if (count($names) > 1 && count($names) !== count(array_unique($names))) {
            $this->setResponse($this->getResponse()->withStatus(400));
            $this->set('errors', ['steps' => 'duplicates in steps names']);

            return false;
        }

        return true;
    }

    /**
     * Activé ou désactivé un circuit
     *
     * @param int $id
     * @param bool|null $state
     *
     * @access public
     *
     * @created 15/10/2020
     * @author Théo GUILLON <theo.guillon@libriciel.coop>
     * @version V2.1.0
     */
    public function toggle($id, $state = null)
    {
//        $this->Droits->assertRecordAuthorized('Formulaire', $id);

        $success = true;
        $this->Workflow->begin();

        $this->Workflow->id = $id;

        $success = $success && $this->Workflow->updateAll([
                'active' => (int) !$state
            ], [
                'id' => $id
            ]) !== false;

        if ($success == true) {
            $this->Workflow->commit();
            $this->Session->setFlash(__d('workflow', 'workflow.flashsuccessCircuitEnregistrer'), 'flashsuccess');

            $this->redirect($this->Referers->get());
        } else {
            $this->Workflow->rollback();
            $this->Session->setFlash(__d('default', 'default.flasherrorEnregistrementErreur'), 'flasherror');
        }
    }
}
