<?php

/**
 * Norme
 *
 * web-DPO : Outil de gestion de vos traitements dans le cadre de la
 * réglementation relative à la protection des données personnelles (RGPD)
 *
 * Copyright (c) Libriciel SCOP (https://www.libriciel.fr/)
 *
 * Licensed under the GNU Affero General Public License version 3 License - AGPL v3
 * For full copyright and license information, please see the "LICENSE" file.
 * Redistributions of files must retain the above copyright notice.
 *
 * @copyright   Copyright (c) Libriciel SCOP (https://www.libriciel.fr/)
 * @link        https://www.libriciel.fr/web-dpo/
 * @since       web-DPO v1.0.0
 * @license     [GNU Affero General Public License version 3](http://www.gnu.org/licenses/agpl-3.0.html) - AGPL v3
 * @version     v1.0.0
 * @package     App.Model
 */

App::uses('AppModel', 'Model');

class Norme extends AppModel {

    public $name = 'Norme';
    
    public $validate = [
        'norme' => [
            'checkUnique' => [
                'rule' => ['checkUnique', ['norme', 'numero']]
            ]
        ],
        'numero' => [
            'inclusiveRange' => [
                'rule' => ['inclusiveRange', 0, 999]
            ],
            'checkUnique' => [
                'rule' => ['checkUnique', ['norme', 'numero']]
            ]
        ]
    ];
    
    /**
     * belongsTo associations
     *
     * @var array
     *
     * @access public
     * @created 07/12/2017
     * @version V1.0.0
     */
    public $belongsTo = [
        'Organisation' => [
            'className' => 'Organisation',
            'foreignKey' => 'organisation_id'
        ]
    ];
    
    public $hasMany = [
        'Fiche' => [
            'className' => 'Fiche',
            'foreignKey' => 'fiche_id'
        ]
    ];
    
    public function beforeValidate($options = []) {
        $result = parent::beforeValidate($options);

        if(true === $result && true === isset($this->data[$this->alias]['numero'])) {
            $numero = Hash::get($this->data, "{$this->alias}.numero");
            $this->data[$this->alias]['numero'] = sprintf('%03d', ltrim($numero, 0));
        }

        return $result;
    }
    
    public function saveFile($data, $id) {
        $success = true;
        $accepted = ['application/pdf'];

        if (isset($data['Norme']['fichier']) && !empty($data['Norme']['fichier'])) {
            $mime = mime_content_type($data['Norme']['fichier']['tmp_name']);

            if (in_array($mime, $accepted) === true) {
                $file = $data['Norme']['fichier'];
                $nameFile = $file['name'];

                // On verifie si le dossier file existe. Si c'est pas le cas on le cree
                $success = $success && create_arborescence_files();

                if (!empty($nameFile) && $success === true) {
                    $extension = pathinfo($nameFile, PATHINFO_EXTENSION);
                    $url = time();
                    $urlFile = $url . '.' . $extension;

                    $success = $success && move_uploaded_file($file['tmp_name'], CHEMIN_FICHIER . NORMES . DS . $urlFile);

                    if ($success === true) {
                        $this->id = $id;
                        $record = [
                            'fichier' => $urlFile,
                            'name_fichier' => $nameFile
                        ];
                        $success = $success && $this->save($record, ['atomic' => false]) !== false;
                    }
                } else {
                    $success = false;
                }
            } else {
                $success = false;
            }
        } else {
            $success = false;
        }

        return $success;
    }
}
