<?php

use Symfony\Component\HttpFoundation\Response;
//use Symfony\Component\Mime\Part\DataPart;
//use Symfony\Component\Mime\Part\Multipart\FormDataPart;
//use Symfony\Component\HttpClient\HttpClient;


/**
 * Base client class
 *
 * @package App\Model\Logic\Http
 */
abstract class ApiClient
{
    protected $http;
    protected $base_url;
    protected $base_api;
    protected $username;
    protected $password;

    /**
     * Client constructor.
     * @param mixed $http The HTTP object
     * @param string $url The URL to query
     * @param string $apiBase The base uri of the API actions
     */
    public function __construct($http, $url, $apiBase = '/')
    {
        $this->http = $http;
        $this->base_url = $url;
        $this->base_api = $apiBase;
    }

    /**
     * Execute the request corresponding to the HTTP method
     *
     * @param string $endpoint The API endpoint
     * @param string $httpMethod The HTTP method
     * @param mixed $urlParams The parameters to be set in the URL
     * @param mixed $bodyParams The parameters to be set in the body request
     * @return mixed The response of the query
     */
    private function execute($endpoint, $httpMethod, $urlParams, $bodyParams = null)
    {
        $url = $this->base_url . '/' . $this->base_api . '/' . $endpoint;

//        if (!empty($urlParams)) {
//            debug($urlParams);
//            $url .= (strpos($url, '?') === false) ? '?' : '&';
//            $url .= is_string($urlParams) ? $urlParams : http_build_query($urlParams);
//        }


        debug($endpoint);
        debug($httpMethod);
        debug($url);
        debug($urlParams);
        debug($bodyParams);
//
//        debug([
//            'headers' => [
//                'Accept' => 'application/json',
//                'Content-Type' => 'application/json'
//            ],
//            'query' => $urlParams,
//            'body' => $bodyParams
//        ]);

        if ($httpMethod === 'GET') {
            $result = $this->http->request($httpMethod, $url);
        } elseif ($httpMethod === 'POST') {
            $result = $this->http->request($httpMethod, $url, [
                'headers' => [
                    'Accept' => 'application/json',
                    'Content-Type' => 'application/json'
//                    'Content-Type' => 'multipart/form-data'
                ],
                'query' => $urlParams,
                'body' => $bodyParams
            ]);

            debug($result);
//        } elseif ($httpMethod === 'PUT') {
//            $result = $this->http->put($url, $bodyParams, ['type' => 'json']);
//        } elseif ($httpMethod === 'PATCH') {
//            $result = $this->http->patch($url, $bodyParams, ['type' => 'json']);
//        } elseif ($httpMethod === 'DELETE') {
//            $result = $this->http->delete($url);
        } else {
            throw new InvalidArgumentException("Unsupported HTTP method provided :  " . $httpMethod);
        }

//        if ($httpMethod === 'GET') {
//            $result = $this->http->get($url);
//        } elseif ($httpMethod === 'POST') {
//            $result = $this->http->post($url, $bodyParams, ['type' => 'json']);
//        } elseif ($httpMethod === 'PUT') {
//            $result = $this->http->put($url, $bodyParams, ['type' => 'json']);
//        } elseif ($httpMethod === 'PATCH') {
//            $result = $this->http->patch($url, $bodyParams, ['type' => 'json']);
//        } elseif ($httpMethod === 'DELETE') {
//            $result = $this->http->delete($url);
//        } else {
//            throw new InvalidArgumentException("Unsupported HTTP method provided :  " . $httpMethod);
//        }

        return $result;
    }

    /**
     * Set credentials for the request
     *
     * @param string $username The login
     * @param string $password The password
     * @return void
     */
    public function setCredentials($username, $password)
    {
        $this->username = $username;
        $this->password = $password;
    }

    /**
     * Send a HTTP get request
     *
     * @param string $endpoint The API endpoint
     * @param mixed $urlParams The parameters to be set in the URL
     * @return Response The response of the query
     */
    protected function get($endpoint, $urlParams)
    {
        return $this->execute($endpoint, 'GET', $urlParams);
    }

    /**
     * Send a HTTP POST request
     *
     * @param string $endpoint The API endpoint
     * @param mixed $urlParams The parameters to be set in the URL
     * @param mixed $bodyParams The parameters to be set in the body request
     * @return Response The response of the query
     */
    protected function post($endpoint, $urlParams, $bodyParams)
    {
        return $this->execute($endpoint, 'POST', $urlParams, $bodyParams);
    }

    /**
     * Send a HTTP PUT request
     *
     * @param string $endpoint The API endpoint
     * @param mixed $urlParams The parameters to be set in the URL
     * @param mixed $bodyParams The parameters to be set in the body request
     * @return Response The response of the query
     */
    protected function put($endpoint, $urlParams, $bodyParams)
    {
        return $this->execute($endpoint, 'PUT', $urlParams, $bodyParams);
    }

    /**
     * Send a HTTP PATCH request
     *
     * @param string $endpoint The API endpoint
     * @param mixed $urlParams The parameters to be set in the URL
     * @param mixed $bodyParams The parameters to be set in the body request
     * @return Response The response of the query
     */
    protected function patch($endpoint, $urlParams, $bodyParams)
    {
        return $this->execute($endpoint, 'PATCH', $urlParams, $bodyParams);
    }

    /**
     * Send a HTTP DELETE request
     *
     * @param string $endpoint The API endpoint
     * @param mixed $urlParams The parameters to be set in the URL
     * @return Response The response of the query
     */
    protected function delete($endpoint, $urlParams)
    {
        return $this->execute($endpoint, 'DELETE', $urlParams);
    }
}
