<?php

/**
 * BanettesHelper
 *
 * web-DPO : Outil de gestion de vos traitements dans le cadre de la
 * réglementation relative à la protection des données personnelles (RGPD)
 *
 * Copyright (c) Libriciel SCOP (https://www.libriciel.fr/)
 *
 * Licensed under the GNU Affero General Public License version 3 License - AGPL v3
 * For full copyright and license information, please see the "LICENSE" file.
 * Redistributions of files must retain the above copyright notice.
 *
 * @copyright   Copyright (c) Libriciel SCOP (https://www.libriciel.fr/)
 * @link        https://www.libriciel.fr/web-dpo/
 * @since       web-DPO v1.0.0
 * @license     [GNU Affero General Public License version 3](http://www.gnu.org/licenses/agpl-3.0.html) - AGPL v3
 * @version     v2.1.0
 * @package     App.View.Helper
 */

App::uses('FormHelper', 'View/Helper');
App::uses('EtatFiche', 'Model');
App::uses('DefaultUrl', 'Default.Utility');
App::uses('DefaultUtility', 'Default.Utility');

class BanettesHelper extends AppHelper {

    public $helpers = [
        'Html',
        'Time',
        'Session',
        'WebcilForm',
        'Form',
        'Autorisation'
    ];
    
    /**
     * @return type
     * 
     * @access public
     * @created 18/07/2017
     * @version V1.0.0
     */
    public function thead() {
        $tr = $this->Html->tableHeaders(
            [
                [__d('pannel', 'pannel.motEtat') => ['class' => 'col-md-1']],
                [__d('pannel', 'pannel.motNomTraitement') => ['class' => 'col-md-3']],
                [__d('pannel', 'pannel.motFinalitePrincipale') => ['class' => 'col-md-3']],
                [__d('pannel', 'pannel.motSynthese') => ['class' => 'col-md-3']],
                [__d('pannel', 'pannel.motActions') => ['class' => 'col-md-2']],
            ]
        );
        return $this->Html->tag('thead', $tr);
    }

    /**
     * Retourne le contenu de la première cellule, en fonction de l'état du
     * traitement, de l'utilisateur connecté et de l'utilisateur lié à l'état du
     * traitement.
     * 
     * @param array $result
     * @return string
     * 
     * @access public
     * @created 18/07/2017
     * @version V1.0.0
     */
    public function etatIcone(array $result) {
        $me = $this->Session->read('Auth.User.id');
        $etat = Hash::get($result, 'EtatFiche.etat_id');
        $user_id = Hash::get($result, 'EtatFiche.user_id');

        switch($etat) {
            case EtatFiche::ENCOURS_REDACTION:
            case EtatFiche::TRAITEMENT_INITIALISE_REDIGER:
                $class = 'fa-pencil-square-o';
                $text = __d('pannel', 'pannel.textEtatEnCoursRedaction');
                break;

            case EtatFiche::ENCOURS_VALIDATION:
                if($me == $user_id) {
                    $class = 'fa-check-square-o';
                    $text = __d('pannel', 'pannel.textEtatEnCoursValidation');                   
                } else {
                    $class = 'fa-clock-o';
                    $text = __d('pannel', 'pannel.textEtatEnAttente');  
                }
                break;

            case EtatFiche::REFUSER:
                $class = 'fa-times fa-danger';
                $text = '<span class="fa-danger">'.__d('pannel', 'pannel.textEtatRefuser').'</span>';
                break;

            case EtatFiche::VALIDER_DPO:
                $class = 'fa-check';
                $text = __d('pannel', 'pannel.textEtatValiderDPO');
                break;

            case EtatFiche::DEMANDE_AVIS:
                $class = 'fa-eye';

                if ($me == $user_id) {
                    $text = __d('pannel', 'pannel.textEtatEnAttenteDemandeAvis');
                } else {
                    $text = __d('pannel', 'pannel.textEtatDemandeAvis');
                }
                break;

            case EtatFiche::ARCHIVER:
                $class = 'fa-lock';
                $text = __d('pannel', 'pannel.textEtatArchiver');
                break;

            case EtatFiche::REPLACER_REDACTION:
                $class = 'fa-pencil-square-o';
                $text = __d('pannel', 'pannel.textEtatReplacerRedaction');
                break;

            case EtatFiche::MODIFICATION_TRAITEMENT_REGISTRE:
                $class = 'fa-pencil-square-o';
                $text = __d('pannel', 'pannel.textEtatModificationTraitementAuRegistre');
                break;

            case EtatFiche::REPONSE_AVIS:
                $class = 'fa-eye';

                if($me == $user_id) {
                    $text = __d('pannel', 'pannel.textEtatRetourDemandeAvis');
                } else {
                    $text = __d('pannel', 'pannel.textEtatConsulte');
                }
                break;

            case EtatFiche::INITIALISATION_TRAITEMENT :
                $class = 'fa-pencil-square-o';
                $text = __d('pannel', 'pannel.textEtatInitialisationTraitement');
                break;

            case EtatFiche::TRAITEMENT_INITIALISE_RECU :
                $class = 'fa-pencil-square-o';
                $text = __d('pannel', 'pannel.textEtatTraitementInitialiseEnAttenteRedaction');
                break;

            default:
                $class = null;
                $text = '';
        }

        return $this->Html->tag(
            'div',
            $this->Html->tag('i', '', ['class' => "fa {$class} fa-3x"])
                .'<br/>'
                .$text,
            ['class' => 'etatIcone']
        );
    }
    
    // -------------------------------------------------------------------------
    
    protected function _url($url) {
        if (false === is_array($url)) {
            $params = Router::parse($url);
        } else {
            $params = $url;
        }

        $params['url'] = true === isset($params['url']) ? $params['url'] : array();
        $params['controller'] = Inflector::underscore($params['controller']);

        return Router::normalize(Router::reverse($params));
    }
    
    protected function _urlToArray($url) {
        if (false === is_array($url)) {
            $params = Router::parse($url);
        } else {
            $params = $url;
        }

        $params['url'] = true === isset($params['url']) ? $params['url'] : array();
        $params['controller'] = Inflector::underscore($params['controller']);

        return $params;
    }
    
    /**
     * Retourne la chaîne de caractères $string dont les occurences de
     * #Model.champ# ont été remplacées par leur valeur extraite depuis $data.
     *
     * @param array $data
     * @param string $string
     * @return string
     */
    public static function evaluateString(array $data, $string) {
        if (strpos($string, '#') !== false) {
            $pattern = '/("#[^#]+#"|\'#[^#]#\'|#[^#]+#)/';
            if (preg_match_all($pattern, $string, $out)) {
                $tokens = $out[0];
                foreach (array_unique($tokens) as $token) {
                    // Pour échapper efficacement les guillemets simples et doubles
                    if ($token[0] === '"') {
                        $escape = '"';
                        $token = trim($token, '"');
                    } else if ($token[0] === "'") {
                        $escape = "'";
                        $token = trim($token, "'");
                    } else {
                        $escape = false;
                    }

                    $token = trim($token, '#');
                    $value = Hash::get($data, $token);

                    if (false !== $escape) {
                        $value = str_replace($escape, "\\{$escape}", $value);
                    }

                    $string = str_replace("#{$token}#", $value, $string);
                }
            }
            $string = preg_replace('/^\/\//', '/', $string);
        }

        return $string;
    }

    /**
     * Retourne le paramètre $mixed dont les occurences de #Model.champ# ont
     * été remplacées par leur valeur extraite depuis $data.
     *
     * @see Hash::get()
     *
     * @param array $data
     * @param string|array $mixed
     * @return string|array
     */
    public static function evaluate(array $data, $mixed) {
        if (is_array($mixed)) {
            $array = array();
            if (!empty($mixed)) {
                foreach ($mixed as $key => $value) {
                    $array[self::evaluateString($data, $key)] = self::evaluate($data, $value);
                }
            }
            return $array;
        }

        return self::evaluateString($data, $mixed);
    }

    //--------------------------------------------------------------------------
    
    protected $_linkClasses = [
        '/fiches/show/#Fiche.id#' => 'fa-eye',
        '/fiches/edit/#Fiche.id#' => 'fa-pencil',
        '/EtatFiches/relaunch/#Fiche.id#' => 'fa-reply',
        '/fiches/genereTraitement/#Fiche.id#' => 'fa-cog',
        '/fiches/downloadFileTraitement/#Fiche.id#' => 'fa-download',
        '/fiches/historique/#Fiche.id#' => 'fa-history',//action virtuelle
        '/fiches/reponse/#Fiche.id#' => 'fa-reply',//action virtuelle
        '#refuser' => 'fa-times',//action virtuelle
        '/fiches/reorienter/#Fiche.id#' => 'fa-exchange',//action virtuelle
        '/fiches/envoyer/#Fiche.id#' => 'fa-paper-plane',//action virtuelle
        '/fiches/delete/#Fiche.id#' => 'fa-trash'
    ];
    
    protected function _button($url, array $result, array $params = []) {
        $params += [
            'confirm' => null,
            'onclick' => null
        ];
       
        $icone = true === isset($this->_linkClasses[$url])
            ? $this->_linkClasses[$url]
            : null;
        
        $parts = $this->_urlToArray($url);
       
        $toggle = '';
        if (true === in_array($icone, ['fa-times', 'fa-trash'])) {
            $span = '<span class="fa '.$icone.' fa-lg fa-danger TOOT"><!----></span>';
            $class = 'btn btn-default-danger bouton'.Inflector::camelize($parts['action']).' btn-sm my-tooltip';
        } elseif ((true === in_array($icone, ['fa-paper-plane']))) {
            $span = '<span class="fa '.$icone.' fa-lg fa-success"><!----></span>';
            $span .= '<span class="caret"><!----></span>';
            $class = 'btn btn-default-success dropdown-toggle btn-sm my-tooltip';
            $toggle = 'dropdown';
        } else {
            $span = '<span class="fa '.$icone.' fa-lg"><!----></span>';
            $class = 'btn btn-default-default bouton'.Inflector::camelize($parts['action']).' btn-sm my-tooltip';
        }

        $link = $this->Html->link(
            $span,
            static::evaluate($result, $url),
            [
                'class' => $class,
                'data-toggle' => $toggle,
                'escapeTitle' => false,
                'title' => __d('pannel', 'pannel.commentaire'.Inflector::camelize($parts['action']).'Traitement'),
                'id' => static::evaluate($result, 'btn'.Inflector::camelize($parts['action']).'#Fiche.id#'),
                'onclick' => self::evaluateString($result, $params['onclick'])
            ],
            $params['confirm']
        );

        return $link;
    }
    
    protected function _buttonReorienter(array $result, array $params = [])
    {
        if (!empty($params['nbUserValideur']) && $params['nbUserValideur'] >= 2) {
            $fiche_id = Hash::get($result, 'Fiche.id');
            $etatFiche_id = Hash::get($result, 'EtatFiche.id');

            return $this->Html->link('<span class="fa fa-exchange fa-lg"><!----></span>', ['#' => '#'], [
                'data-id' => $fiche_id,
                'data-fiche' => $etatFiche_id,
                'escape' => false,
                'data-toggle' => 'modal',
                'data-target' => '#modalReorienter',
                'class' => 'btn btn-default-default btn_ReorienterTraitement  btn-sm my-tooltip',
                'title' => __d('pannel', 'pannel.commentaireReorienterTraitement')
            ]);
        }
    }
    
    protected function _menuEnvoyerTraitement($banette, array $result, array $params = []) {
        $fiche_id = Hash::get($result, 'Fiche.id');
        $etatFiche_id = Hash::get($result, 'EtatFiche.id');
        $menu= '';

        $fiche_typedeclaration = 'false';
        
        foreach ($result['Valeur'] as $valeur) {
            if ($valeur['champ_name'] === 'typedeclaration' && !empty($valeur['champ_name'])) {
                $fiche_typedeclaration = 'true';
            }
        }

        if ($this->Autorisation->isDpo() && $banette == 'initialisation') {
            $title = __d('pannel', 'pannel.textEnvoyerRedacteur');

            // Envoyer pour validation
            $menu .= $this->Html->tag(
                'li',
                $this->Html->link($title, ['#' => '#'], [
                    'role' => 'menuitem',
                    'tabindex' => -1,
                    'escape' => false,
                    'data-toggle' => 'modal',
                    'data-target' => '#modalSendRedaction',
                    'data-id' => $fiche_id,
                    'data-fiche' => $etatFiche_id,
                    'class' => 'btn_envoyerValideur'
                ]),
                ['role' => 'presentation']
            );
        } else {
            if (!empty($params['nbUserConsultant'])) {
                // Envoyer pour consultation
                $menu .= $this->Html->tag(
                    'li',
                    $this->Html->link(__d('pannel', 'pannel.textEnvoyerConsultation'), ['#' => '#'], [
                        'role' => 'menuitem',
                        'tabindex' => -1,
                        'escape' => false,
                        'data-toggle' => 'modal',
                        'data-target' => '#modalEnvoieConsultation',
                        'data-id' => $fiche_id,
                        'data-fiche' => $etatFiche_id,
                        'class' => 'btn_envoyerConsultation'
                    ]),
                    ['role' => 'presentation']
                );
            }

            if (!empty($params['nbUserValideur'])) {
                if ($banette == 'encours_redaction') {
                    $titreSendValidation = __d('pannel', 'pannel.textEnvoyerValidation');
                } else {
                    $titreSendValidation = __d('pannel', 'pannel.textValiderEnvoyerValidation');
                }

                // Envoyer pour validation
                $menu .= $this->Html->tag(
                    'li',
                    $this->Html->link($titreSendValidation, ['#' => '#'], [
                        'role' => 'menuitem',
                        'tabindex' => -1,
                        'escape' => false,
                        'data-toggle' => 'modal',
                        'data-target' => '#modalEnvoieValidation',
                        'data-id' => $fiche_id,
                        'data-fiche' => $etatFiche_id,
                        'class' => 'btn_envoyerValideur'
                    ]),
                    ['role' => 'presentation']
                );
            }

            // Envoyer au DPO ou insèrer au registre
            if ($this->Autorisation->isDpo() && $result['EtatFiche']['etat_id'] == EtatFiche::ENCOURS_VALIDATION) {
                $menu .= $this->Html->tag(
                    'li',
                    $this->Html->link(__d('pannel', 'pannel.textValiderInsererRegistre'), ['#' => '#'], [
                        'role' => 'menuitem',
                        'tabindex' => '-1',
                        'data-toggle' => 'modal',
                        'data-target' => '#modalValidDpo',
                        'data-type' => $fiche_typedeclaration,
                        'data-id' => $fiche_id,
                        'class' => 'btn-insert-registre',
                        'data-norme-id' => Hash::get($result, 'Fiche.norme_id'),
                    ]),
                    []
                );
            } else {
                if ($banette == 'encours_redaction') {
                    $title = __d('pannel', 'pannel.textEnvoyerDPO');
                } else {
                    $title = __d('pannel', 'pannel.textValiderEnvoyerDPO');
                }

                $menu .= $this->Html->tag(
                    'li',
                    $this->Html->link($title, [
                        'controller' => 'etatFiches',
                        'action' => 'dpoValid',
                        $fiche_id
                    ], [
                        'role' => 'menuitem',
                        'tabindex' => '-1'
                    ]),
                    ['role' => 'presentation']
                );
            }
        }

        $span = $this->Html->tag(
            'ul',
            $menu,
            ['class' => 'dropdown-menu', 'role' => 'menu']
        );
        
        return $span;
    }
    
    /**
     * Groupe de boutons correspondant à la banette 
     * "Mes déclarations en cours de rédaction"
     * 
     * @param array $result
     * @param array $params
     * @return string
     * 
     * @access protected
     *
     * @created 19/07/2017
     * @version V1.0.0
     *
     * @modified 11/12/2019
     * @version V2.0.0
     * @author Théo GUILLON <theo.guillon@libriciel.coop>
     *
     * @modified 30/09/2020
     * @version V2.1.0
     * @author Théo GUILLON <theo.guillon@libriciel.coop>
     */
    protected function _buttonsEncoursRedaction(array $result, array $params = []) {
        $fiche_id = Hash::get($result, 'Fiche.id');
        $etat = Hash::get($result, 'EtatFiche.etat_id');
        $traitement_name = $this->_getOutilnom(Hash::extract($result, 'Valeur.{n}'));

        $btnGroups = $this->_button('/fiches/genereTraitement/#Fiche.id#', $result)
            . $this->_button('/fiches/show/#Fiche.id#', $result)
            . $this->_button('/fiches/edit/#Fiche.id#', $result)
            . $this->_button('/fiches/historique/#Fiche.id#', $result, ['onclick' => "$('#listeValidation{$fiche_id}').toggle(); return false;"]);

        if ($etat !== EtatFiche::TRAITEMENT_INITIALISE_RECU) {
            $btnGroups .= $this->_button('/fiches/envoyer/#Fiche.id#', $result)
                . $this->_menuEnvoyerTraitement('encours_redaction', $result, $params)
                .$this->_button('/fiches/delete/#Fiche.id#', $result, ['confirm' => __d('pannel', 'pannel.confirmationSupprimerTraitement') . $traitement_name . ' " ?']);
        }

        return $btnGroups;
    }

    /**
     * Groupe de boutons correspondant à la banette "Mes déclarations en attente"
     * 
     * @param array $result
     * @param array $params
     * @return string
     * 
     * @access protected
     *
     * @created 19/07/2017
     * @version V1.0.0
     *
     * @modified 30/09/2020
     * @version V2.1.0
     * @author Théo GUILLON <theo.guillon@libriciel.coop>
     */
    protected function _buttonsAttente(array $result, array $params = [])
    {
        $fiche_id = Hash::get($result, 'Fiche.id');

        return $this->_button('/fiches/genereTraitement/#Fiche.id#', $result)
            . $this->_button('/fiches/show/#Fiche.id#', $result)
            . $this->_button('/fiches/historique/#Fiche.id#', $result, ['onclick' => "$('#listeValidation{$fiche_id}').toggle(); return false;"])
            . $this->_buttonReorienter($result, $params);
    }
    
    /**
     * Groupe de boutons correspondant à la banette "Mes déclarations refusées"
     * 
     * @param array $result
     * @param array $params
     * @return string
     * 
     * @access protected
     *
     * @created 19/07/2017
     * @version V1.0.0
     *
     * @modified 30/09/2020
     * @version V2.1.0
     * @author Théo GUILLON <theo.guillon@libriciel.coop>
     */
    protected function _buttonsRefuser(array $result, array $params = [])
    {
        $fiche_id = Hash::get($result, 'Fiche.id');
        $traitement_name = $this->_getOutilnom(Hash::extract($result, 'Valeur.{n}'));
        
        return $this->_button('/fiches/genereTraitement/#Fiche.id#', $result)
            . $this->_button('/fiches/show/#Fiche.id#', $result)
            . $this->_button('/EtatFiches/relaunch/#Fiche.id#', $result)
            . $this->_button('/fiches/historique/#Fiche.id#', $result, ['onclick' => "$('#listeValidation{$fiche_id}').toggle(); return false;"])
            . $this->_button('/fiches/delete/#Fiche.id#', $result, ['confirm' => __d('pannel', 'pannel.confirmationSupprimerTraitement') . $traitement_name . ' " ?']);
    }
    
    /**
     * Groupe de boutons correspondant à la banette "Mes traitements validés et insérés au registre"
     * 
     * @param array $result
     * @param array $params
     * @return string
     * 
     * @access protected
     *
     * @created 19/07/2017
     * @version V1.0.0
     *
     * @modified 30/09/2020
     * @version V2.1.0
     * @author Théo GUILLON <theo.guillon@libriciel.coop>
     */
    protected function _buttonsArchives(array $result, array $params = []) {
        $fiche_id = Hash::get($result, 'Fiche.id');

        if ($result['EtatFiche']['etat_id'] == EtatFiche::VALIDER_DPO || $result['EtatFiche']['etat_id'] == EtatFiche::MODIFICATION_TRAITEMENT_REGISTRE){
            $action = 'genereTraitement';
        } else if ($result['EtatFiche']['etat_id'] == EtatFiche::ARCHIVER) {
            $action = 'downloadFileTraitement';
        }
        
        return $this->_button('/fiches/show/#Fiche.id#', $result)
            .$this->_button('/fiches/'.$action.'/#Fiche.id#', $result)    
            .$this->_button('/fiches/historique/#Fiche.id#', $result, ['onclick' => "$('#listeValidation{$fiche_id}').toggle(); return false;"]);
    }
    
    /**
     * Groupe de boutons correspondant à la banette "Etat des traitements passés en ma possession"
     * 
     * @param array $result
     * @param array $params
     * @return string
     * 
     * @access protected
     * @created 19/07/2017
     * @version V1.0.0
     *
     * @modified 30/09/2020
     * @version V2.1.0
     * @author Théo GUILLON <theo.guillon@libriciel.coop>
     */
    protected function _buttonsConsulte(array $result, array $params = []) {
        $fiche_id = Hash::get($result, 'Fiche.id');
        $etatFiche_etatId = Hash::get($result, 'EtatFiche.etat_id');
        $etatFiche_userId = Hash::get($result, 'EtatFiche.user_id');
        $traitement_name = Hash::get($result, 'Valeur.0.valeur');
        
        $reorienter = null;
        $remiseRedaction = null;
        if ($this->Autorisation->authorized(ListeDroit::VALIDER_TRAITEMENT, $this->Session->read('Droit.liste')) && $etatFiche_etatId == EtatFiche::ENCOURS_VALIDATION && $this->Session->read('Auth.User.id') != $etatFiche_userId) {
            $reorienter = $this->_buttonReorienter($result, $params);
        } elseif ($this->Autorisation->authorized(ListeDroit::INSERER_TRAITEMENT_REGISTRE, $this->Session->read('Droit.liste'))) {
            if ($etatFiche_etatId == EtatFiche::REFUSER) {
                $remiseRedaction = $this->_button('/EtatFiches/relaunch/#Fiche.id#', $result, ['confirm' => __d('pannel', 'pannel.confirmationReapproprierTraitement') . $traitement_name . ' " ?']);
            }
        }

        return $this->_button('/fiches/genereTraitement/#Fiche.id#', $result)
            . $this->_button('/fiches/show/#Fiche.id#', $result)
            . $this->_button('/fiches/historique/#Fiche.id#', $result, ['onclick' => "$('#listeValidation{$fiche_id}').toggle(); return false;"])
            . $reorienter
            . $remiseRedaction;
    }
    
    /**
     * Groupe de boutons correspondant à la banette "Traitements reçus pour validation"
     * 
     * @param array $result
     * @param array $params
     * @return string
     * 
     * @access protected
     *
     * @created 19/07/2017
     * @version V1.0.0
     *
     * @modified 30/09/2020
     * @version V2.1.0
     * @author Théo GUILLON <theo.guillon@libriciel.coop>
     */
    protected function _buttonsRecuValidation(array $result, array $params = []) {
        $fiche_id = Hash::get($result, 'Fiche.id');
        
        return $this->_button('/fiches/genereTraitement/#Fiche.id#', $result)
            . $this->_button('/fiches/show/#Fiche.id#', $result)
            . $this->_button('/fiches/edit/#Fiche.id#', $result)
            . $this->_button('/fiches/historique/#Fiche.id#', $result, ['onclick' => "$('#listeValidation{$fiche_id}').toggle(); return false;"])
            . $this->_button('/fiches/envoyer/#Fiche.id#', $result)
            . $this->_menuEnvoyerTraitement('recuValidation', $result, $params)
            . $this->_button('#refuser', $result, ['onclick' => "$('#commentaireRefus{$fiche_id}').toggle(); return false;"]);
    }
    
    /**
     * Groupe de boutons correspondant à la banette "Traitements reçus pour consultation"
     * 
     * @param array $result
     * @param array $params
     * @return string
     * 
     * @access protected
     *
     * @created 19/07/2017
     * @version V1.0.0
     *
     * @modified 30/09/2020
     * @version V2.1.0
     * @author Théo GUILLON <theo.guillon@libriciel.coop>
     */
    protected function _buttonsRecuConsultation(array $result, array $params = []) {
        $fiche_id = Hash::get($result, 'Fiche.id');

        return $this->_button('/fiches/genereTraitement/#Fiche.id#', $result)
            . $this->_button('/fiches/show/#Fiche.id#', $result)
            . $this->_button('/fiches/historique/#Fiche.id#', $result, ['onclick' => "$('#listeValidation{$fiche_id}').toggle(); return false;"])
            . $this->_button('/fiches/reponse/#Fiche.id#', $result, ['onclick' => "$('#commentaireRepondre{$fiche_id}').toggle(); return false;"]);
    }
    
    /**
     * 
     * @param type $banette
     * @param array $result
     * @param array $params
     * @return type
     * 
     * @access protected
     * @created 19/07/2017
     * @version V1.0.0
     */
    protected function _buttons($banette, array $result, array $params = []) {
        $method = '_buttons'.Inflector::camelize($banette);
        $content = null;

        if(true === method_exists($this, $method)) {
            $content = $this->{$method}($result, $params);
        }

        return $this->Html->tag('div',$content,['class' => 'btn-group']);
    }

    /**
     * Nouvelle ligne dans le tableau de la banette avec l'historique du
     * traitement.
     * 
     * @param array $result
     * @param array $params
     * @return type
     * 
     * @access protected
     * @created 19/07/2017
     * @version V1.0.0
     */
    protected function _rowHistorique(array $result, array $params = []) {
        $fiche_id = Hash::get($result, 'Fiche.id');

        $parcours = $this->requestAction(['controller' => 'Pannel', 'action' => 'parcours', $fiche_id]);
        $historique = $this->requestAction(['controller' => 'Pannel', 'action' => 'getHistorique', $fiche_id]);
        

        $options = ['class' => 'listeValidation', 'id' => "listeValidation{$fiche_id}"];

        return $this->Html->tableCells(
            [
                [
                    ['', []],
                    [$this->_View->element('parcours', ['parcours' => $parcours]), ['class' => 'tdleft', 'colspan' => 3]],
                    [$this->_View->element('historique', ['historique' => $historique, 'id' => $fiche_id]), ['class' => 'tdleft']],
                    ['', []]
                ]   
            ],
            $options,
            $options,
            false
        );
    }
    
    /**
     * Nouvelle ligne dans le tableau de la banette pour répondre dans le cas 
     * d'un refus ou d'une consultation
     * 
     * @param array $result
     * @param array $params
     * @return type
     * 
     * @access protected
     * @created 19/07/2017
     * @version V1.0.0
     */
    protected function _rowReponse(array $result, array $params = []) {
        $etatFiche_id = Hash::get($result, 'EtatFiche.id');
        $etatFiche_previousUserId = Hash::get($result, 'EtatFiche.previous_user_id');
        $fiche_id = Hash::get($result, 'Fiche.id');

        $options = ['class' => 'commentaireRepondre', 'id' => "commentaireRepondre{$fiche_id}"];

        $content = $this->WebcilForm->create('EtatFiche', [
            'url' => ['action' => 'answerAvis'],
            'autocomplete' => 'off',
            'inputDefaults' => ['div' => false],
            'class' => 'form-horizontal',
            'novalidate' => 'novalidate'
        ]);

        $content .= $this->WebcilForm->inputs([
            'EtatFiche.commentaireRepondre' => [
                'id' => 'commentaireRepondre',
                'label' => [
                    'text' =>  __d('pannel', 'pannel.champCommentaireRepondre'),
                    'class' => 'col-md-4'
                ],
                'class' => 'form-control reponseStartTinyMCE',
                'between' => '<div class="col-md-12">',
                'after' => '</div>',
                'required' => true,
                'placeholder' => false,
                'type' => 'textarea'
            ]
        ]);
        
        // Champs cachés
        $content .= $this->WebcilForm->hidden('etatFiche', ['value' => $etatFiche_id]);
        $content .= $this->WebcilForm->hidden('previousUserId', ['value' => $etatFiche_previousUserId]);
        $content .= $this->WebcilForm->hidden('ficheNum', ['value' => $fiche_id]);
        
        // Groupe de bouton
        $content .= $this->WebcilForm->buttons(['Cancel', 'Send']);
        
        $content .= $this->WebcilForm->end();

        return $this->Html->tableCells(
            [
                [
                    ['', []],
                    [$content, ['class' => 'tdleft col-md-12', 'colspan' => 3]],
                    ['', []]
                ]   
            ],
            $options,
            $options,
            false
        );
    }
    
    protected function _rowRefuser(array $result, array $params = []) {
        $etatFiche_id = Hash::get($result, 'EtatFiche.id');
        $fiche_id = Hash::get($result, 'Fiche.id');

        $options = ['class' => 'commentaireRefus', 'id' => "commentaireRefus{$fiche_id}"];

        $content = $this->WebcilForm->create('EtatFiche', [
            'url' => ['action' => 'refuse'],
            'autocomplete' => 'off',
            'inputDefaults' => ['div' => false],
            'class' => 'form-horizontal',
            'novalidate' => 'novalidate'
        ]);
        
        $content .= $this->WebcilForm->inputs([
            'EtatFiche.commentaireRepondre' => [
                'id' => 'commentaireRepondre',
                'label' => [
                    'text' =>  __d('pannel', 'pannel.textExpliquezRaisonRefus'),
                    'class' => 'col-md-4'
                ],
                'between' => '<div class="col-md-12">',
                'after' => '</div>',
                'required' => true,
                'placeholder' => false,
                'type' => 'textarea'
            ]
        ]);
        
        // Champs cachés
        $content .= $this->WebcilForm->hidden('ficheNum', ['value' => $fiche_id]);
        $content .= $this->WebcilForm->hidden('etatFiche', ['value' => $etatFiche_id]);
        
        // Groupe de bouton
        $content .= $this->WebcilForm->buttons(['Cancel', 'Refuse']);
        
        $content .= $this->WebcilForm->end();

        return $this->Html->tableCells(
            [
                [
                    ['', []],
                    [$content, ['class' => 'tdleft', 'colspan' => 3]],
                    ['', []]
                ]   
            ],
            $options,
            $options,
            false
        );
    }

    /**
     * 
     * 
     * @param type $banette
     * @param array $result
     * @param array $params
     * @return type
     * 
     * @access protected
     * @created 19/07/2017
     * @version V1.0.0
     */
    protected function _rowSupplementaire($banette, array $result, array $params = []) {
        if('recuConsultation' === $banette) {
            return $this->_rowHistorique($result, $params) . $this->_rowReponse($result, $params);
        } elseif ('recuValidation' === $banette) {
            return $this->_rowHistorique($result, $params).$this->_rowRefuser($result, $params);
        } else {
            return $this->_rowHistorique($result, $params);
        }
    }

    /**
     * 
     * @param array $results
     * @param array $params
     * @return type
     * 
     * @access protected
     * @created 18/07/2017
     * @version V1.0.0
     */
    protected function _banette(array $results, array $params = []) {
        $params += [
            'limit' => false,
            'link' => false
        ];
        
        $traitement = Inflector::camelize($params['action']);

        if (true === empty($results)) {
            $h3 = $this->Html->tag('h3', __d('pannel', "pannel.aucunTraitement{$traitement}"));
            $span = null;
            $content = $this->Html->tag('div', $h3, ['class' => 'text-center']);
        } else {
            if (false !== $params['limit'] && $params['count'] >= $params['limit']) {
                $span = $this->Html->tag(
                    'span',
                    $this->Html->link('<span class="fa fa-eye fa-fw"><!----></span>' . __d('pannel', "pannel.btnVoirTraitement{$traitement}"), [
                            'controller' => 'pannel',
                            'action' => $params['action'],
                                ], [
                            'class' => 'btn btn-default-primary',
                            'escapeTitle' => false,
                    ]),
                    ['class' => 'pull-right']
                );
            } else {
                $span = null;
            }

            $thead = $this->thead();
            
            $rows = '';
            foreach ($results as $result) {
                if (!empty($result['Norme']['id'])) {
                    $title = Hash::get($params['enumsNormes'], "Norme.norme.{$result['Norme']['norme']}");
                    $abbr = $this->Html->tag('abbr', $result['Norme']['norme'], ['title' => $title]);
                    
                    $titleDescription = strip_tags($result['Norme']['description'], '<br />');
                    $numeroNorme = sprintf('%03d', $result['Norme']['numero']);
                    $abbrNumero = $this->Html->tag('abbr', $numeroNorme, ['title' => $titleDescription]);
                    
                    $norme = ' ' . $abbr . '-' . $abbrNumero . ' : ' . $result['Norme']['libelle'];
                    
                } else {
                    $norme = __d('registre', 'registre.textAucuneNormeDefini') ;
                }
                
                if ($result['Fiche']['coresponsable'] == true) {
                    $infoCoresponsable = '<strong>' . '<strong>' . __d('registre', "registre.textTableauCoresponsable") . '</strong>'. $this->Html->tag('i', 'oui', ['class' => 'fa fa-check fa-lg fa-success']) . '</strong>' . ' ' . '<br/>';
                } else {
                    $infoCoresponsable = '';
                }

                if ($result['Fiche']['soustraitance'] == true) {
                    $infoSousTraitance = '<strong>' . '<strong>' . __d('registre', "registre.textTableauSousTraitance") . '</strong>'. $this->Html->tag('i', 'oui', ['class' => 'fa fa-check fa-lg fa-success']) . '</strong>' . ' ' . '<br/>';
                } else {
                    $infoSousTraitance = '';
                }
                
//                if (!empty($result['Fiche']['soustraitantid'])) {
//                    $infoSoustraitant = '<strong>' . '<strong>' . __d('registre', "registre.textTableauSoustraitant") . '</strong>'. $this->Html->tag('i', 'oui', ['class' => 'fa fa-check fa-lg fa-success']) . '</strong>' . ' ' . '<br/>';
//                } else {
//                    $infoSoustraitant = '';
//                }

                $valeursTraitement = Hash::extract($result,'Valeur.{n}');
                foreach ($valeursTraitement as $valeurTraitement) {
                    if ($valeurTraitement['champ_name'] === 'outilnom') {
                        $nomTraitement = $valeurTraitement['valeur'];
                    }

                    if ($valeurTraitement['champ_name'] === 'finaliteprincipale') {
                        $finalitePrincipaleTraitement = $valeurTraitement['valeur'];
                    }

                    if ($valeurTraitement['champ_name'] === 'declarantservice') {
                        $declarantServiceTraitement = $valeurTraitement['valeur'];
                    }
                }

                $rows .= $this->Html->tableCells(
                    [
                        [
                            // Etat du traitement
                            [$this->etatIcone($result), ['class' => 'tdleft col-md-1']],
                            // Nom du traitement
                            [$nomTraitement, ['class' => 'tdleft']],
                            // Finalité principale
                            [nl2br($finalitePrincipaleTraitement), ['class' => 'tdleft']],
                            // Synthèse
                            [
                                '<strong>' . __d('pannel', "pannel.motCreee") . '</strong>'. $result['User']['prenom'] . ' ' . $result['User']['nom'] .'<br/>'.
                                '<strong>' . __d('registre', "registre.textTableauDateCreation") . '</strong>'. $this->Time->format($result['Fiche']['created'], FORMAT_DATE_HEURE).'<br/>'.
                                '<strong>' . __d('pannel', "pannel.motDerniereModification") . '</strong>'. $this->Time->format($result['Fiche']['modified'], FORMAT_DATE_HEURE).'<br/>'.
                                '<strong>' . __d('registre', "registre.textTableauServiceDeclarant") . '</strong>'. $declarantServiceTraitement . '<br/>' .
                                '<strong>' . __d('pannel', "pannel.textNorme") . '</strong>' . $norme . '<br/>'.
                                $infoCoresponsable.
                                $infoSousTraitance,
//                                $infoSoustraitant,
                                ['class' => 'tdleft']
                            ],
                            // Actions
                            [$this->_buttons($params['action'], $result, $params),['class' => 'tdleft']],
                        ]
                    ]
                )
                .$this->_rowSupplementaire($params['action'], $result);
            }

            $tbody = $this->Html->tag('tbody', $rows);

            $content = $this->Html->tag('table', $thead.$tbody, ['class' => 'table']);
            
            // @todo: ajouter des actions si besoin ("Créer un traitement" dans "Mes déclarations en cours de rédaction")
        }

        $title = $this->Html->tag(
            'h3',
            sprintf(__d('pannel', "pannel.traitement{$traitement}"), Hash::get($params, 'count')).$span,
            ['class' => 'panel-title']
        );
            
        $head = $this->Html->tag(
            'div',
            $this->Html->tag(
                'div',
                $this->Html->tag('div', $title, ['class' => 'col-md-12']),
                ['class' => 'row']
            ),
            ['class' => 'panel-heading']
        );
        
        $body = $this->Html->tag('div', $content, ['class' => 'panel-body panel-body-custom']);

        $link = false === empty($params['link']) ? $params['link'] : '';
        
        return $this->Html->tag('div', $head.$body.$link, ['class' => 'panel panel-primary']);
    }
    
    /**
     * 
     * 
     * @param array $banette
     * @param array $params
     * @return type
     * 
     * @access public
     * @created 18/07/2017
     * @version V1.0.0
     */
    public function encoursRedaction(array $banette, array $params = []) {
        $params += [
            'action' => 'encours_redaction',
            'count' => $banette['count'],
            'link' => $this->Html->tag(
                'div',
                $this->Html->tag(
                    'div',
                    $this->Html->link('<i class="fa fa-plus fa-lg"><!----></i>' . ' ' . __d('pannel', 'pannel.btnCreerTraitement'), ['#' => '#'], [
                        'escape' => false,
                        'data-toggle' => 'modal',
                        'data-target' => '#modalAddTraitement',
                        'class' => 'btn btn-default-primary'
                    ]),
                    ['class' => 'col-md-12 text-center']
                ),
                ['class' => 'row bottom10']
            ),
            'nbUserValideur' => $banette['nbUserValideur'],
            'nbUserConsultant' => $banette['nbUserConsultant'],
            'enumsNormes' => $banette['enumsNormes']
        ];
        
        return $this->_banette($banette['results'], $params);
    }
    
    /**
     * 
     * @param array $banette
     * @param array $params
     * @return type
     * 
     * @access public
     * @created 18/07/2017
     * @version V1.0.0
     */
    public function attente(array $banette, array $params = []) {
        $params += [
            'action' => 'attente',
            'count' => $banette['count'],
            'nbUserValideur' => $banette['nbUserValideur'],
            'enumsNormes' => $banette['enumsNormes']
        ];

        return $this->_banette($banette['results'], $params);
    }
    
    /**
     * 
     * @param array $banette
     * @param array $params
     * @return type
     * 
     * @access public
     * @created 18/07/2017
     * @version V1.0.0
     */
    public function recuValidation(array $banette, array $params = []) {
        $params += [
            'action' => 'recuValidation',
            'count' => $banette['count'],
            'nbUserValideur' => $banette['nbUserValideur'],
            'nbUserConsultant' => $banette['nbUserConsultant'],
            'enumsNormes' => $banette['enumsNormes']
        ];

        // @todo
        $cle = [];
        foreach ($banette['results'] as $key => $etatTraitement) {
            if ($etatTraitement['EtatFiche']['etat_id'] == 2 && $etatTraitement['EtatFiche']['actif'] == true) {
                $tampon = $key+1;
                if (array_key_exists($tampon,$banette['results']) == true) {
                    if ($banette['results'][$key+1]['EtatFiche']['etat_id'] == 6 &&$banette['results'][$key+1]['EtatFiche']['actif'] == true){
                        unset($banette['results'][$key]);
                    }
                }
            }
        }
                
        return $this->_banette($banette['results'], $params);
    }
    
    /**
     * 
     * @param array $banette
     * @param array $params
     * @return type
     * 
     * @access public
     * @created 18/07/2017
     * @version V1.0.0
     */
    public function recuConsultation(array $banette, array $params = []) {
        $params += [
            'action' => 'recuConsultation',
            'count' => $banette['count'],
            'enumsNormes' => $banette['enumsNormes']
        ];

        return $this->_banette($banette['results'], $params);
    }
    
    /**
     * 
     * @param array $banette
     * @param array $params
     * @return type
     * 
     * @access public
     * @created 18/07/2017
     * @version V1.0.0
     */
    public function refuser(array $banette, array $params = []) {
        $params += [
            'action' => 'refuser',
            'count' => $banette['count'],
            'enumsNormes' => $banette['enumsNormes']
        ];

        return $this->_banette($banette['results'], $params);
    }
    
    /**
     * 
     * @param array $banette
     * @param array $params
     * @return type
     * 
     * @access public
     * @created 18/07/2017
     * @version V1.0.0
     */
    public function archives(array $banette, array $params = []) {
        $params += [
            'action' => 'archives',
            'count' => $banette['count'],
            'enumsNormes' => $banette['enumsNormes']
        ];

        return $this->_banette($banette['results'], $params);
    }
    
    /**
     * 
     * @param array $banette
     * @param array $params
     * @return type
     * 
     * @access public
     * @created 18/07/2017
     * @version V1.0.0
     */
    public function consulte(array $banette, array $params = []) {
        $params += [
            'action' => 'consulte',
            'count' => $banette['count'],
            'nbUserValideur' => $banette['nbUserValideur'],
            'enumsNormes' => $banette['enumsNormes']
        ];

        return $this->_banette($banette['results'], $params);
    }

    /**
     *
     *
     * @param array $banette
     * @param array $params
     * @return type
     *
     * @access public
     * @created 10/12/2019
     * @version V2.0.0
     * @author Théo GUILLON <theo.guillon@libriciel.coop>
     */
    public function initialisationTraitement(array $banette, array $params = []) {
        $params += [
            'action' => 'initialisation',
            'count' => $banette['count'],
            'link' => $this->Html->tag(
                'div',
                $this->Html->tag(
                    'div',
                    $this->Html->link('<i class="fa fa-plus fa-lg"><!----></i>' . ' ' . __d('pannel', 'pannel.btnInitialisationTraitement'), ['#' => '#'], [
                        'escape' => false,
                        'data-toggle' => 'modal',
                        'data-target' => '#modalInitialisationTraitement',
                        'class' => 'btn btn-default-primary'
                    ]),
                    ['class' => 'col-md-12 text-center']
                ),
                ['class' => 'row bottom10']
            ),
            'enumsNormes' => $banette['enumsNormes']
        ];

        return $this->_banette($banette['results'], $params);
    }

    /**
     * Groupe de boutons correspondant à la banette
     * "Mes déclarations en cours d'initialisation"
     *
     * @param array $result
     * @param array $params
     * @return string
     *
     * @access protected
     * @created 11/12/2019
     * @version V2.0.0
     */
    protected function _buttonsInitialisation(array $result, array $params = [])
    {
        $fiche_id = Hash::get($result, 'Fiche.id');

        $traitement_name = $this->_getOutilnom(Hash::extract($result, 'Valeur.{n}'));

        return $this->_button('/fiches/show/#Fiche.id#', $result)
            .$this->_button('/fiches/edit/#Fiche.id#', $result)
            .$this->_button('/fiches/historique/#Fiche.id#', $result, ['onclick' => "$('#listeValidation{$fiche_id}').toggle(); return false;"])
            .$this->_button('/fiches/envoyer/#Fiche.id#', $result)
            .$this->_menuEnvoyerTraitement('initialisation', $result, $params)
            .$this->_button('/fiches/delete/#Fiche.id#', $result, ['confirm' => __d('pannel', 'pannel.confirmationSupprimerTraitement') . $traitement_name . ' " ?']);

    }

    /**
     * Retourne la valeur "outilnom" (nom du traitement)
     *
     * @param array $valeursTraitement
     *
     * @access protected
     * @created 11/12/2019
     * @version V2.0.0
     * @author Théo GUILLON <theo.guillon@libriciel.coop>
     */
    protected function _getOutilnom($valeursTraitement)
    {
        $traitement_name = '';

        foreach ($valeursTraitement as $key => $valeur) {
            if ($valeur['champ_name'] === 'outilnom') {
                $traitement_name = $valeur['valeur'];
            }
        }

        return ($traitement_name);
    }

    /**
     *
     * @param array $banette
     * @param array $params
     * @return type
     *
     * @access public
     * @created 12/12/2019
     * @version V2.0.0
     * @author Théo GUILLON <theo.guillon@libriciel.coop>
     */
    public function allTraitement(array $banette, array $params = []) {
        $params += [
            'action' => 'allTraitements',
            'count' => $banette['count'],
            'enumsNormes' => $banette['enumsNormes'],
        ];

        return $this->_banette($banette['results'], $params);
    }

    /**
     * Groupe de boutons correspondant à la banette "Toutes les déclarations de l'entité"
     *
     * @param array $result
     * @param array $params
     * @return string
     *
     * @access protected
     * @created 12/12/2019
     * @version V2.0.0
     * @author Théo GUILLON <theo.guillon@libriciel.coop>
     */
    protected function _buttonsAllTraitements(array $result, array $params = []) {
        $fiche_id = Hash::get($result, 'Fiche.id');

        return $this->_button('/fiches/show/#Fiche.id#', $result)
            .$this->_button('/fiches/historique/#Fiche.id#', $result, [
                'onclick' => "$('#listeValidation{$fiche_id}').toggle(); return false;"
            ]);
    }
}
