<?php

/**
 * La classe WebdpoMenuHelper simplifie l'écriture d'un menu pour web-dpo
 * en fournissant des méthodes facilitant la construction de menus sous forme de
 * liste non ordonnées (ul) imbriquées.
 */
class WebdpoMenuHelper extends AppHelper {

    public $helpers = ['Html'];

    /**
     * Permet de construire un menu à plusieurs niveaux.
     *
     * Cette méthode ajoute la possibilité de:spécifier la balise qui sera
     * utilisée pour construire les éléments inactifs "parents" d'éléments
     * actifs, d'ajouter un attribut title aux éléments et de désactiver un
     * élément (et ses sous-éléments) en plus des permissions.
     *
     * @param array $items Les éléments du menu
     * @fixme
     * @return string
     */
    public function main($items, $disabledTag = 'a', $level = 0, $extraTopUlClass = null)
    {
        $return = '';

        foreach ($items as $key => $item) {
            if (is_numeric($key) && $item === 'divider') {
                $return .= $this->Html->tag('li', '<!---->', ['class' => 'divider']);
            } elseif (is_numeric($key) && is_string($item)) {
                $return .= $this->Html->tag('li', $item, ['class' => 'dropdown-header']);
            } elseif (!isset($item['disabled']) || !$item['disabled']) {
                $sub = $item;
                $title = ( isset($sub['title']) ? $sub['title'] : false );
                $class = ( isset($sub['class']) ? $sub['class'] : false );
                // @todo Faire plus propre, on n'a peut-être pas que les data-* à gérer
                $extra = [];
                foreach ($sub as $subKey => $subValue) {
                    if (strpos($subKey, 'data-') === 0) {
                        $extra[$subKey] = $subValue;
                        unset($sub[$subKey]);
                    }
                }
                unset($sub['url'], $sub['disabled'], $sub['title'], $sub['class']);

                $sub = $this->main($sub, $disabledTag, $level+1);

                $content = '';
                $htmlOptions = ['title' => $title];

                // Cas array('url' => array('controller' => 'moncontroller', 'action' => 'monaction'))
                if (isset($item['url']['controller']) && $item['url']['controller'] !== '/') {
                    $controllerName = $item['url']['controller'];
                    $actionName = isset($item['url']['action']) ? $item['url']['action'] : 'index';
                }

                // Cas array('url' => '/moncontroller/monaction')
                elseif (isset($item['url']) && is_string($item['url']) && preg_match('/^\/(?:([\w]+)\/([\w]+)|([\w]+))/', $item['url'], $matches)) {
                    $controllerName = $matches[1];
                    $actionName = isset($matches[2]) ? $matches[2] : 'index';
                } else {
                    $controllerName = '/';
                    $actionName = '/';
                }

                if (isset($item['url']) && ( $item['url'] != '#' )) {
                    $url = $item['url'];
                    if (is_array($url)) {
                        $url += ['plugin' => null];
                    }
                    
                    $content .= $this->Html->link(
                        $this->Html->tag('i', '<!---->', ['class' => $class === false ? false : "fa {$class} fa-fw"])
                        . ' ' . $key,
                        $url,
                        $extra + ['escape'=>false]
                    ). $sub;
                } else if (!empty($sub)) {
                    $htmlOptions = [];

                    if ($disabledTag == 'a') {
                        $htmlOptions['href'] = '#';
                    }

                    if ($level === 0) {
                        $caret = $this->Html->tag('span', '<!---->', ['class' => 'caret']);
                        $content .= $this->Html->tag($disabledTag, $key . $caret, ['class' => 'dropdown-toggle', 'data-toggle' => 'dropdown'] + $htmlOptions) . $sub;
                    } else {
                        $content .= $this->Html->tag($disabledTag, $key, $htmlOptions) . $sub;
                    }
                }

                $liClass = ( empty($sub) ? '' : ($level === 0 ? 'dropdown' : 'dropdown-submenu') );
                $return .= empty($content) ? '' : $this->Html->tag('li', $content, ['class' => $liClass, 'title' => $title]);
            }
        }
        $ulParams = $level === 0
            ? ['class' => "nav navbar-nav {$extraTopUlClass}"]
            : ['class' => 'dropdown-menu', 'role' => 'menu'];
        return empty($return) ? '' : $this->Html->tag('ul', $return, $ulParams);
    }

}
